program CAOServer;
{$APPTYPE CONSOLE}

(*
 * To compile, make sure you have the BorCon2003.dll in this same directory, 
 * generated from the BorCon2003.cs (which is located in the ..\c#\assemblies
 * directory).
 *
 * dccil -luBorCon2003 CAOServer.dpr 
 *
 * Use this server application with the CAOClient.
 *
 *)

uses
  RickRoss.BorCon2003, // imported from BorCon2003.dll
  System.Runtime.Remoting,
  System.Runtime.Remoting.Channels.Http,
  System.Runtime.Remoting.Channels,
  System.Runtime.Remoting.Lifetime;  

type
  // notice that this class now implements ISponsor as well
  TMyRemoteObject = class( MarshalByRefObject, IExample1, ISponsor )
  private
    FValue : integer;
  public
    constructor Create; overload;
    constructor Create( anInitialValue : integer); overload;
    function  InitializeLifetimeService : System.Object; override;
    function  Renewal( lease : ILease ) : TimeSpan;
    procedure SetValue( value : integer );
    function  GetValue : integer;
  end;

  TMyRemoteObjectFactory = class( MarshalByRefObject, IExample1Factory )
  public
    function GetNewInstance : IExample1; overload;
    function GetNewInstance( anInitialValue : integer ) : IExample1; overload;
  end;

constructor TMyRemoteObject.Create; 
begin
  inherited Create;
  FValue := 1;
end;

constructor TMyRemoteObject.Create( anInitialValue : integer); 
begin
  inherited Create;
  FValue := anInitialValue;
end;

function  TMyRemoteObject.InitializeLifetimeService : System.Object; 
var
  lease : Ilease;
begin
  writeln('Server: Initializing lifetime service...');
  lease := inherited InitializeLifetimeService() as ILease;
  if (lease.CurrentState = LeaseState.Initial) then
  begin
    lease.InitialLeaseTime := TimeSpan.FromMinutes(1);
    lease.Register( self );
  end;
  Result := lease;
end;

function TMyRemoteObject.Renewal(lease : ILease ) : TimeSpan;
begin
  // renew this lease for another minute...
  // normally, some logic would be in here to determine if it
  // was time to renew the lease...
  writeln('Request for renewal...');
  lease.Renew( TimeSpan.FromMinutes( 1 ) );
end;

procedure TMyRemoteObject.SetValue( value : integer );
begin
  writeln('Server: Setting value to ', value);
  FValue := value;
end;

function  TMyRemoteObject.GetValue : integer;
begin
  writeln('Server: getting value ', FValue );
  Result := FValue;
end;


function TMyRemoteObjectFactory.GetNewInstance : IExample1;
begin
  writeln('Server: Creating a new instance with default value ');
  Result := TMyRemoteObject.Create;
end;

function TMyRemoteObjectFactory.GetNewInstance( anInitialValue : integer ) : IExample1;
begin
  writeln('Server: Creating a new instance with value: ', anInitialValue);
  Result := TMyRemoteObject.Create( anInitialValue );
end;

var
  aChannel : HttpChannel;

begin
  writeln('RemoteServer (SingleCall) starting...');
  aChannel := HttpChannel.Create( 2003 ); // port number
  writeln('Server: Registering channel...');
  ChannelServices.RegisterChannel( aChannel );
  writeln('Server: Registering type...');
  RemotingConfiguration.RegisterWellKnownServiceType( typeof(TMyRemoteObjectFactory), 
         'BorCon2003RemoteObjectFactory.soap',
         WellKnownObjectMode.Singleton );
  writeln('Press return to shut down the server...');
  readln;  
end.
