program RemotePrimeClient;
{$APPTYPE CONSOLE}

(*
 * To compile the client, make sure that the RemotePrimeCalc.dll and
 * RemotePrimeCalc.dcpil are located in this same directory. RemotePrimeCalc is
 * generated from the RemotePrimeCalc.dpk package. Complete build instructions
 * can be found in the buildall.bat file.
 *
 * The client connects to the RemotePrimeServer application.
 *
 * Usage:
 *    RemotePrimeClient ServerURI ClientName
 *        where ServerURI is similar to: http://localhost:2003/PrimeCalcServer.soap
 *          and ClientName is an identifier like "one"
 *
 *)

{%DelphiDotNetAssemblyCompiler 'c:\winnt\microsoft.net\framework\v1.1.4322\System.Runtime.Remoting.dll'}

uses
  System.Collections,
  System.Runtime.Remoting,
  System.Runtime.Remoting.Channels.Http,
  System.Runtime.Remoting.Channels,
  RickRoss.RemotePrimeCalc;  // imported from RemotePrimeCalc.dll

function IsPrime(n : Int64) : boolean;
var
  i : Int64;
  stop : Int64;
  e : extended;

begin
  Result := true;
  e := n;
  i := 2;
  stop := trunc(sqrt(e));
  while (i <= stop) do
  begin
    if i*(n div i) = n then
    begin
      Result := false;
      break;
    end;
    inc(i);
  end;
end;

function CalcPrimes( data : PrimeNumberData ) : integer;
var
  start : Int64;
  stop  : Int64;
  tmpArray : array of Int64;

begin
  Result := 0;
  start := data.StartAt;
  stop  := data.StopAt;
  writeln('Client: finding primes between ',start,' and ',stop);
  SetLength( tmpArray, 0 );

  while (start <= stop) do
  begin
    if IsPrime(start) then
    begin
      inc(Result);
      SetLength( tmpArray, Result);
      tmpArray[Result-1] := start;
    end;
    inc(start);
  end;
  data.PrimeNumbers := tmpArray;
  writeln('client: Found ', Result,' prime numbers between ',data.StartAt,' and ',data.StopAt);
end;

var
  iPrimeSvr : IPrimeNumberServer;
  data      : PrimeNumberData;

begin
  if ParamCount < 2 then
  begin
    writeln('RemotePrimeClient server_uri clientname');
    writeln('  where server_uri looks similar to: http://localhost:2003/PrimeCalcServer.soap');
    Exit;
  end;

  writeln('Remote Prime client is reading the configuration file...');
  RemotingConfiguration.Configure( 'RemotePrimeClient.exe.config');

  writeln('obtaining reference to server...');
  writeln('Server URI is ', ParamStr(1) );

  iPrimeSvr := Activator.GetObject(
             typeof(IPrimeNumberServer), ParamStr(1) ) as
             IPrimeNumberServer;

  if not assigned(iPrimeSvr) then
  begin
    writeln('Unable to obtain server interface!');
    Exit;
  end;
  
  writeln('Client ', ParamStr(2) ,' obtained server interface to ', ParamStr(1) );

  while (true) do
  begin
    data := iPrimeSvr.ReadyForWork( paramStr(2) );
    if data.TimeToStop then
       break;
    CalcPrimes(data);
    iPrimeSvr.PrimeNumbers( data );
  end;

  writeln('RemotePrimeClient is done!');
end.

