program SerializableExample;
{$APPTYPE CONSOLE}

//
// To compile:
//   dccil SerializableExample
//
// Running:
//   SerializeableExample  (with any parameter) persists the object to the disk
//   SerializeableExample  (no parameters) reads the persisted object from the disk
//

uses
  System.IO,
  System.Runtime.Serialization,
  System.Runtime.Serialization.Formatters.Binary;

type
  [Serializable]
  TSerializeThis = class
  strict private
    FStrictPrivate : integer;
  public
    constructor Create; overload;
    constructor Create( a : integer ); overload;
    procedure ShowValues; 
  end;

constructor TSerializeThis.Create;
begin
  inherited Create;
  FStrictPrivate   := 100;
end;

constructor TSerializeThis.Create( a : integer );
begin
  inherited Create;
  FStrictPrivate   := a;
end;

procedure TSerializeThis.ShowValues; 
begin
  writeln('Value: ', FStrictPrivate);
end;

const
  FILENAME = 'se.dat';

var
  myobj     : TSerializeThis;
  fs        : FileStream;
  formatter : IFormatter;
  
begin
  formatter := BinaryFormatter.Create();

  if ParamCount >= 1 then
  begin
    writeln('Creating file...');
    myobj := TSerializeThis.Create( 1 );
    myobj.ShowValues;
    fs := FileStream.Create( FILENAME, FileMode.Create, FileAccess.Write, FileShare.None );
    try
      formatter.Serialize( fs, myobj );
    finally
      fs.Close();
    end;
  end
  else
  begin
    writeln('Opening file...');
    fs := FileStream.Create( FILENAME, FileMode.Open, FileAccess.Read, FileShare.Read);
    try
      myobj := TSerializeThis( formatter.Deserialize(fs) );
    finally
      fs.Close();
    end;
    myobj.ShowValues;
  end;
end.