program SingletonLiveForeverServer;
{$APPTYPE CONSOLE}

(*
 * To compile, make sure you have the BorCon2003.dll in this same directory, 
 * generated from the BorCon2003.cs (which is located in the ..\c#\assemblies
 * directory).
 *
 * dccil -luBorCon2003 SingletonLiveForeverServer.dpr 
 *
 * Use this server application with either the Delphi or C# ExampleClient1 client
 *
 *)

uses
  RickRoss.BorCon2003, // imported from BorCon2003.dll
  System.Runtime.Remoting,
  System.Runtime.Remoting.Channels.Http,
  System.Runtime.Remoting.Channels,
  System.Runtime.Remoting.Lifetime;  

type
  TMyRemoteObject = class( MarshalByRefObject, IExample1, ISponsor )
  private
    FValue : integer;
  public
    function  InitializeLifetimeService : System.Object; override;
    function  Renewal( lease : ILease ) : TimeSpan;
    procedure SetValue( value : integer );
    function  GetValue : integer;
  end;

function  TMyRemoteObject.InitializeLifetimeService : System.Object; 
var
  lease : Ilease;
begin
  //
  // another alternative is to return nil from this method. 
  // that will also make the RemoteObject live forever
  //
  writeln('Server: Initializing lifetime service...');
  lease := inherited InitializeLifetimeService() as ILease;
  if (lease.CurrentState = LeaseState.Initial) then
  begin
    lease.InitialLeaseTime := TimeSpan.FromMinutes(1);
    lease.Register( self );
  end;
  Result := lease;
end;

function TMyRemoteObject.Renewal(lease : ILease ) : TimeSpan;
begin
  // renew this lease for another 5 minutes...
  writeln('Request for renewal...');
  lease.Renew( TimeSpan.FromMinutes( 1 ) );
end;

procedure TMyRemoteObject.SetValue( value : integer );
begin
  writeln('Server: Setting value to ', value);
  FValue := value;
end;

function  TMyRemoteObject.GetValue : integer;
begin
  writeln('Server: getting value ', FValue );
  Result := FValue;
end;

var
  aChannel : HttpChannel;

begin
  writeln('RemoteServer (Singleton) starting...');
  aChannel := HttpChannel.Create( 2003 ); // port number
  writeln('Server: Registering channel...');
  ChannelServices.RegisterChannel( aChannel );
  writeln('Server: Registering type...');
  RemotingConfiguration.RegisterWellKnownServiceType( typeof(TMyRemoteObject), 
         'BorCon2003RemoteObject.soap',
         WellKnownObjectMode.Singleton );
  writeln('Press return to shut down the server...');
  readln;  
end.
